%==========================================================================
%
%             ADNSS: DYNAMIC FACTOR MODEL WITH CORRELATED ERRORS
%
%
%             Model:    YY(t) = ZZ*S(t) ;
%                       S(t)  = D + PHI*S(t-1)  + e(t)
%
%                       e(t), ~ N(0, SIG)
%==========================================================================

clc;clear;
tic;

adnss = path;
path('Data',path);
path('Moment',path);
path('Prior',path);
path('Output',path);
path('Main Files',path);
path('Kalman Filter',path);
path('Gauss Files',path);
path('Sims Code',path);
path('Posterior Mode',path);

%=========================================================================
%                      METROPOLIS-HASTINGS SAMPLER
%=========================================================================
disp('                                                                  ')
disp('   DYNAMIC FACTOR MODEL WITH CORRELATED ERRORS                    ')
disp('                                                                  ')


% import prior specification and load data
prior_data
import_data
%import_simulated_data
bound

YY       = 4*YYY(:,1:2) ;
data_std = std(YYY);
data_var = var(YY);

[T,nv]   = size(YY);

load paraest.csv; load hess.csv

para     = paraest;
npara    = length(para);
H        = hess;
sigscale = H;
fcn      = @(x1) objfcn(x1,pshape,pmean,pstdd,pmask,pmaskinv,pfix,lubound,YY);

%=========================================================================
%                            CONFIGURE
%=========================================================================

nblocks  = 1;
nsim     = 50000;
nburn    = 0*nsim;
cc0      = 2;
cc       = 0.10;  
counter  = 0;

%=========================================================================
%                    METROPOLIS-HASTINGS ALGORITHM
%========================================================================= 
%--------------------------------------------------------------------------     
%           initial draw for Random Walk Metropolis Algorithm
%--------------------------------------------------------------------------


lbd = lubound;
check = 0;
while check<1
    
    paraold = mvnrnd(paraest, cc0*sigscale,1)';
    paraold = paraold.*pmaskinv + pfix.*pmask;
    [postold likeold At_old AtF_old sGGold] = fcn(paraold); 
    if postold > -1E6
        check = 1;
    end
end

para_old   = [paraold;sGGold];

%--------------------------------------------------------------------------     
%           block loop for Random Walk Metropolis Algorithm
%--------------------------------------------------------------------------

for iblock = 1:nblocks
        
    % storages 
    parasim  = zeros(nsim,npara+1);       % parameter draws
    likesim  = zeros(nsim,1);           % likelihood
    postsim  = zeros(nsim,1);           % posterior probability
    rej      = zeros(nsim,1);           % rej = 1: rejected
    f_sim    = zeros(nsim,T);
    eGDPsim  = zeros(nsim,T);
    eGDIsim  = zeros(nsim,T);
    ff_sim   = zeros(nsim,T);
    kg_sim   = zeros(nsim,2);
    % with-in block loop 
    for j = 1:nsim
    
        % report 
        counter = counter +1; 
    
        if counter==100
        
        disp(['  BLOCKS:   ',              num2str(iblock)]             );
        disp('                                                         ');
        disp(['  POSTERIOR DRAWS:   ',     num2str(j)]                  );
        disp('                                                         ');
        disp(['  REMAINING DRAWS:   ',     num2str(nsim-j)]             );
        disp('                                                         ');        
        disp(['  REJECTION RATE:   ',      num2str(mean(rej(1:j)))]     );
        disp('                                                         ');
        disp(['  SUMMARY - MU & RHO:   ',num2str(median(parasim(1:j-1,1:2)))]);
        disp('                                                         ');
        disp(['  SUMMARY - VAR-COVAR:   ',  num2str(median(parasim(1:j-1,3:end)))]     );
        disp('                                                         ');
        disp(['  SUMMARY - POSTERIOR:   ',    num2str(median(postsim(1:j-1,:)))]     );
        disp('                                                         ');
        counter=0;
        
        end      
        
        % propose   
        genacc = 0;
        while genacc < 1
            paranew = mvnrnd(paraold, cc*sigscale,1)';
            paranew = paranew.*pmaskinv + pfix.*pmask;
            par1    = paranew;
            
            % check boundary conditions
            genacc  = all(par1 < lbd(:,2) & par1 > lbd(:,1));
        end
        
        
        % evaluate at the new candidate 
        [postnew likenew At_new AtF_new sGGnew KGnew] = fcn(paranew);            
        para_new   = [paranew; sGGnew];
        
        % evaluate at the old candidate              
        [postold likeold At_old AtF_old sGGold KGold] = fcn(paraold);  
        para_old   = [paraold; sGGold];
        
        r = min([1 exp( postnew - postold)]);            
        
        if rand > r 
           % reject proposed jump             
           rej(j) = 1;
        else
           % accept proposed jump          
           paraold  = paranew;
           postold  = postnew;
           likeold  = likenew;
           At_old   = At_new;
           AtF_old  = AtF_new;
           para_old = para_new;
           KGold    = KGnew;
        end
        
        % store simulated variables
        f_sim(j,:)    = At_old(:,1)'; 
        eGDPsim(j,:)  = At_old(:,2)'; 
        eGDIsim(j,:)  = At_old(:,3)'; 
        ff_sim(j,:)   = AtF_old(:,1)';
        likesim(j,:)  = likeold;
        postsim(j,:)  = postold;           
        parasim(j,1:2)     = para_old(1:2)';
        parasim(j,4:end-1) = para_old(4:end-1)';
        parasim(j,3)       = para_old(end)';
        parasim(j,end)     = para_old(3);
        kg_sim(j,:) = KGold(end,:);   
        
    end       % with-in block  

end           % block 


%=========================================================================
%                    DISPLAY ESTIMATION RESULTS
%=========================================================================

x = sum(recession,2)/2;
w = recession(:,2) - recession(:,1);


    
%% compute posterior values    
% order: [mu rho sig_eta_eta sig_etaE sig_etaI sig_EE sig_EI sig_II]
[mu50 mu05 mu95]           = moment(parasim(nburn+1:end,1));
[rr50 rr05 rr95]           = moment(parasim(nburn+1:end,2));
[ee50 ee05 ee95]           = moment(parasim(nburn+1:end,3));
[eE50 eE05 eE95]           = moment(parasim(nburn+1:end,4));
[eI50 eI05 eI95]           = moment(parasim(nburn+1:end,5));
[EE50 EE05 EE95]           = moment(parasim(nburn+1:end,6));
[EI50 EI05 EI95]           = moment(parasim(nburn+1:end,7));
[II50 II05 II95]           = moment(parasim(nburn+1:end,8));
[zz50 zz05 zz95]           = moment(parasim(nburn+1:end,9));
[pp50 pp05 pp95]           = moment(postsim(nburn+1:end,:));
[kg50 kg05 kg95]           = moment(kg_sim(nburn+1:end,:));

rgEsim = 100*kg_sim(:,1)./(kg_sim(:,1)+kg_sim(:,2));
rgIsim = 100*kg_sim(:,2)./(kg_sim(:,1)+kg_sim(:,2));

[rE50 rE05 rE95]           = moment(rgEsim(nburn+1:end,:));
[rI50 rI05 rI95]           = moment(rgIsim(nburn+1:end,:));

etasim = zeros(nsim,T-1);
for jj=1:nsim
    etasim(jj,:) = f_sim(jj,2:end)-parasim(jj,1)*(1-parasim(jj,2))-parasim(jj,2)*f_sim(jj,1:end-1);
end

[f50 f05 f95]              = moment(f_sim(nburn+1:end,:)); 
[e150 e105 e195]           = moment(eGDPsim(nburn+1:end,:)); 
[e250 e205 e295]           = moment(eGDIsim(nburn+1:end,:)); 
[eta50 eta05 eta95]        = moment(etasim(nburn+1:end,:));


YHAT50 = zeros(T,nv);
YHAT95 = zeros(T,nv);
YHAT05 = zeros(T,nv);

for t=1:T
YHAT50(t,:) = [1 1 ]*f50(t) + [e150(t) e250(t) ];
YHAT95(t,:) = [1 1 ]*f95(t) + [e195(t) e295(t) ];
YHAT05(t,:) = [1 1 ]*f05(t) + [e105(t) e205(t) ];
end


postdist50 = [mu50;rr50;ee50;eE50;eI50;EE50;EI50;II50;zz50;pp50;kg50';rE50';rI50'];
postdist95 = [mu95;rr95;ee95;eE95;eI95;EE95;EI95;II95;zz95;pp95;kg95';rE95';rI95'];
postdist05 = [mu05;rr05;ee05;eE05;eI05;EE05;EI05;II05;zz05;pp05;kg05';rE05';rI05'];


%% display median values and figures

vm_summary_mh
%vm_figures_mh

save results_81.mat

path=adnss;

disp('                                                                 ');
disp(['                      ELAPSED TIME:   ', num2str(toc)]          );
disp('                                                                 ');

elapsedtime=toc;
    
    