%==========================================================================
%
%             ADNSS: DYNAMIC FACTOR MODEL WITH CORRELATED ERRORS
%
%
%             Model:    YY(t) = ZZ*S(t) ;
%                       S(t)  = D + PHI*S(t-1)  + L*e(t)
%
%                       e(t), ~ N(0, L*L')
%==========================================================================


%=========================================================================
%                             HOUSEKEEPING
%=========================================================================

tic
close all
clear all
clc

adnss = path;
path('Data',path);
path('Moment',path);
path('Prior',path);
path('Output',path);
path('Main Files',path);
path('Kalman Filter',path);
path('Gauss Files',path);
path('Sims Code',path);
path('Posterior Mode',path);

% import prior specification and load data
prior_data
import_data
%import_simulated_data
bound

YY       = 4*YYY(:,1:2) ;
data_std = std(YYY);
data_var = var(YY);

[T,nv]   = size(YY);

%=========================================================================
%                       MAXIMIZE POSTERIOR DENSITY
%=========================================================================

% set optimization routine environment 
nit  = 1000;        % maximum number of iteration 
crit = 1E-12;        % tolerance 
H0   = (1e-2)*eye(npara);

para=[7.05590162;
0.156960171;
0.81;
-1.161652554;
-0.817941655;
5.153910134;
2.383592931;
2.680617054];
    
% use Sims's optimization procedure 
[ fh, xh, gh, H, itct, fcount, retcodeh ] ...
= csminwel( 'objfcn0', para, H0, [], crit, nit, pshape,pmean,pstdd,pmask,pmaskinv,pfix,lubound,YY);


paraest = real(xh);
paraest = paraest.*pmaskinv + pfix.*pmask;


dlmwrite('Posterior Mode/paraest.csv',paraest, 'precision', '%.6f', 'newline', 'pc')    
dlmwrite('Posterior Mode/hess.csv',H, 'precision', '%.6f', 'newline', 'pc')    



path=adnss;
 
disp(['         ELAPSED TIME:   ', num2str(toc)]);

elapsedtime=toc;