function   [shatnew,signew,lh,yhat,fin,kgain,yforc]=kf_mod(y,H,shat,sig,G,M)
% function [shatnew,signew,lh,yhat,fin,kgain,yforc]=kf_mod(y,H,shat,sig,G,M)
% 
% This is Chris Sims's KF with a couple of added outputs 
% 1) The Kalman Gain and F^(-1) matrices obtained using the 
%     Generalized Inverse are part of the output
% s(t) = G*s(t-1) + R*n(t)  V( n(t) ) = Q 
% then   M=R*Chol(Q)'=R( CQ') 
%        M*M'=R*( CQ'*CQ )*R'
% 
% Added the forecast errors 
% SHATNEW = s(t)|t 
% SIGNEW  = P(t)|t 
% LH      = Log Likelihood, w/o Normalizing cosntant 
% YHAT    = Forecast error  
% FIN     = Inverse matrix of the forecast error 
% KGAIN   = Kalman Gain 
% YFORC   = One step ahead forecast for Y 
%
% ============================
lh=zeros(1,2);
omega=G*sig*G'+M*M';
[uo do vo]=svd(omega);
[u d v]=svd(H*uo*sqrt(do));
first0=min(find(diag(d)<1e-12));
if isempty(first0),first0=min(size(H))+1;end
u=u(:,1:first0-1);
v=v(:,1:first0-1);
d=diag(d);d=diag(d(1:first0-1));
spred = G*shat; 
fac=vo*sqrt(do);
yforc = H*spred; 
yhat=y-yforc; 
fhalf=(v/d)*u'; 
fin=fhalf'*fhalf; 
ferr=fhalf*yhat;
lh(1)=-.5*ferr'*ferr;
lh(2)=-sum(log(diag(d)));
%kgain=G*omega*H'*fin
kgain=G*fac*fhalf; 
shatnew=fac*ferr+spred;
signew=fac*(eye(size(v,1))-v*v')*fac';
lh=sum(lh)-0.5*size(y,1)*log(2*pi); 